Pattern

Struct Pattern 

pub struct Pattern {
    pub chance: f32,
    pub priority: f32,
    pub before: CellGrid,
    pub after: CellGrid,
}
A pattern consists both of a grid of cells to search for and a grid of cells to replace it with.

The before pattern may contain wildcards * to match any character. The after pattern may contain wildcards * to not mutate that cell and simply keep its previous value.

Whenever a pattern matches, the attribute might randomly be discarded instead of being applied. The chance attribute describes the likelihood of the pattern being applied without discard, i.e. any value over 1.0 means the pattern will always be applied when it matches.

If multiple patterns are applicable within a time step, the one with higher priority will always be applied first. Only if no cell concerning the second pattern has been mutated, the second pattern will apply also.

use cellumina::rule::Rule;
let rule = cellumina::rule::PatternRule::from_patterns(
    &[
        cellumina::rule::Pattern{
            chance: 1.0,
            priority: 1.0,
            before: grid::grid![['X'][' ']],
            after: grid::grid![[' ']['X']],
        },
        cellumina::rule::Pattern{
            chance: 1.0,
            priority: 0.5,
            before: grid::grid![[' ', 'X']['X', ' ']],
            after: grid::grid![['X', 'X'][' ', ' ']],
        },
    ],
    cellumina::rule::BoundaryBehaviour::Symbol('_'),
    cellumina::rule::BoundaryBehaviour::Symbol('_'),
);

let mut grid = grid::grid![[' ', 'X']['X', ' '][' ', ' ']];
rule.transform(&mut grid);
assert_eq!(grid, grid::grid![[' ', ' '][' ', 'X']['X', ' ']]);
rule.transform(&mut grid);
assert_eq!(grid, grid::grid![[' ', ' '][' ', ' ']['X', 'X']]);

let rule2 = cellumina::rule::PatternRule::from(rule.to_string().as_str());

grid = grid::grid![[' ', 'X']['X', ' '][' ', ' ']];
rule2.transform(&mut grid);
assert_eq!(grid, grid::grid![[' ', ' '][' ', 'X']['X', ' ']]);
rule2.transform(&mut grid);
assert_eq!(grid, grid::grid![[' ', ' '][' ', ' ']['X', 'X']]);

Fields

chance: f32

The chance for the pattern to apply on a match.

priority: f32

The priority of this pattern over others.

before: CellGrid

The cell pattern to search for.

after: CellGrid

The cell pattern it should be replaced with.